//  (C) Copyright Gennadiy Rozental 2001.
//  Distributed under the Boost Software License, Version 1.0.
//  (See accompanying file LICENSE_1_0.txt or copy at
//  http://www.boost.org/LICENSE_1_0.txt)

//  See http://www.boost.org/libs/test for the library home page.
//
//! @file
//! @brief Deprecated header.
//! @deprecated Use @c boost/test/unit_test.hpp instead.
// ***************************************************************************

#ifndef BOOST_TEST_AUTO_UNIT_TEST_HPP_071894GER
#define BOOST_TEST_AUTO_UNIT_TEST_HPP_071894GER

#include <boost/config/header_deprecated.hpp>
BOOST_HEADER_DEPRECATED( "<boost/test/unit_test.hpp>" )

#include <boost/test/unit_test.hpp>

#endif // BOOST_TEST_AUTO_UNIT_TEST_HPP_071894GER
