//=======================================================================
// (c) Copyright Juergen Hunold 2008
// Use, modification and distribution is subject to the Boost Software
// License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)
//=======================================================================

#ifndef BOOST_DEPRECATED_INCLUDE_EDMONDS_KARP_MAX_FLOW_HPP
#define BOOST_DEPRECATED_INCLUDE_EDMONDS_KARP_MAX_FLOW_HPP

#if defined(_MSC_VER) || defined(__BORLANDC__) && !defined(__clang__) || defined(__DMC__)
#pragma message( \
    "Warning: This header is deprecated. Please use: boost/graph/edmonds_karp_max_flow.hpp")
#elif defined(__GNUC__) || defined(__HP_aCC) || defined(__SUNPRO_CC) \
    || defined(__IBMCPP__) || defined(__BORLANDC__)
#warning \
    "This header is deprecated. Please use: boost/graph/edmonds_karp_max_flow.hpp"
#endif

#include <boost/graph/edmonds_karp_max_flow.hpp>

#endif // BOOST_DEPRECATED_INCLUDE_EDMONDS_KARP_MAX_FLOW_HPP
