// Copyright (c) 2021 Klemens D. Morgenstern
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#define BOOST_TEST_IGNORE_SIGCHLD
#define BOOST_TEST_MODULE process_v2_test
#include <boost/test/included/unit_test.hpp>

#include <boost/process/v2/src.hpp>
