//  include.hpp  ---------------------------------------------------------------//

//  Copyright 2011 Vicente J. Botet Escriba

//  Distributed under the Boost Software License, Version 1.0.
//  See http://www.boost.org/LICENSE_1_0.txt

#ifndef BOOST_RATIO_INCLUDE_HPP
#define BOOST_RATIO_INCLUDE_HPP

#include <boost/ratio/ratio.hpp>
#include <boost/ratio/ratio_io.hpp>
#ifdef BOOST_RATIO_EXTENSIONS
#include <boost/ratio/mpl/rational_constant.hpp>
#endif
#endif  // BOOST_RATIO_INCLUDE_HPP
